'
' -----[ Title ]------------------------------------------------------
'
' File......: SNAP 1.BAS
' Purpose...: Turn a LEDs on and off on PortC
' Author....: Mike Eitel
' Version...: 1.00AVR
' MCU.......: AVR 8515
' Started...: 991002
' Updated...: 991002
'
' -----[ Program Description ]----------------------------------------
'
' This program shows how to implement the S.N.A.P protocol in BASCOM
' and is an simple example to turn a LED ON or OFF. It uses STK200
' Power Line Modems to communicate over the power lines (mains). This
' example uses 16-bit CRC-CCITT as error detection method which gives
' secure data transfer.
'
' If the node is addressed by another node (PC or another MCU) and DB1
' contains 0 it will turn the LED off. If DB1 contains any other value
' (i.e <> 0) it will turn the LED on.
'
' The packet structure is defined in the received packets first two
' bytes (HDB2 and HDB1). The following packet structure is used.
'
' DD=01     - 1 Byte destination address
' SS=01     - 1 Byte source address
' PP=00     - No protocol specific flags
' AA=01     - Acknowledge is required
' C=0       - Command mode not supported
' EEE=100   - 16-bit CRC-CCITT
' NNNN=0001 - 1 Byte data
'
' Overview of header definition bytes (HDB2 and HDB1)
'
'        HDB2              HDB1
' +-----------------+-----------------+
' | D D S S P P A A | C E E E N N N N |
' +-----------------+-----------------+
'
' ************************************************************************
' This File is based on a program from Christer Johansson HTH and I do not
' intent to touch any of his rights !!!
' So for correct use of his copyrights please inform as marked below!
' ************************************************************************
' For more information about S.N.A.P see URL below.
' http://www.hth.com/snap/
'
' -----[ Disclaimer ]-------------------------------------------------
'
' This example is offered on an "AS IS" basis, no warranty expressed
' or implied. The programers disclaim liability of any damages associated with
' the use of the hardware or software described herein. You use it on
' your own risk. We are not able to provide any free support.
'
' Copyright (c) 1998-1999 HTH all rights reserved
'
' -----[ Revision History ]-------------------------------------------
'
' 980529 - Version 1.00    Initial release                  Christer Johansson HTH
' 980919 - Version 1.01    Revised for S.N.A.P Version 0.91 Christer Johansson HTH
' 990130 - Version 1.02    Revised for S.N.A.P Version 1.00 Christer Johansson HTH
' 991002 - Version 1.00AVR S.N.A.P Ver 1.00 for AVR 8515    Mike Eitel
' -----[ Aliases ]----------------------------------------------------
'
Ledport Alias Portc                                 ' LED pins

' -----[ Constants ]--------------------------------------------------
'
Const Preamble = &B01010011                         ' Preamble byte
Const Sync = &B01010100                             ' Synchronisation byte
Const Crcpoly = &H1021                              ' CRC-CCITT
Const Chdb2 = &B01010001                            ' &B01010001 DD=01     - 1 Byte destination address
'                                                                SS=01     - 1 Byte source address
'                                                                PP=00     - No protocol specific flags
'                                                                AA=01     - Acknowledge is required

Const Chdb1r = &B01000001                           ' &B01000001 C=1 / 0   - Command mode / not supported
Const Chdb1w = &B11000001                           '            Eee = 100 -  16 -bit Crc -ccitt
'                                                                NNNN=0001 - 1 Byte data
Const Myaddress = &H04                              ' Address for this node (1-255)
' -----[ Variables ]--------------------------------------------------
'
Dim Temp1 As Byte                                   ' Temporary Variable
Dim Temp2 As Byte                                   ' Temporary Variable
Dim Temp_crc As Word                                ' Temporary Variable
Dim Crc As Word                                     ' CRC word variable
Dim Crc2 As Byte                                    ' CRC hi byte variable
Dim Crc1 As Byte                                    ' CRC lo byte variable
Dim Hdb2 As Byte                                    ' Header Definition Byte 2
Dim Hdb1 As Byte                                    ' Header Definition Byte 1
Dim Db1 As Byte                                     ' Packet data
Dim Sab1 As Byte                                    ' What node sent this packet
Dim Dab1 As Byte                                    ' What node should have this paket
Dim Commands As Byte                                ' Byte to recognize the requested commands
Dim Hisaddress As Byte                              ' Byte containing the last requester adress
Dim Datas As Byte                                   ' The received datas

' -----[ Initialization ]---------------------------------------------
'
' Define serial communication parameters
$baud = 9600
$crystal = 4000000

Config Porta = Input
Config Portb = Output
Config Portc = Output

Config Timer1 = Timer , Prescale = 256

Const Timer1_reload = &H1FFF
Timer1 = Timer1_reload

'define ISR when timer 1 overflows
On Timer1 Timer1_isr

Enable Interrupts
Tccr1a = &B00010011                                 'dammned trick to get the OC1B output blinking


 '------[ Program ]----------------------------------------------------
'
Print Chr(preamble) ; Chr(preamble) ; Chr(preamble)
_start:

 ' Wait for data on serialport
   Inputbin Temp1

 ' If received data is SYNC byte read next seven bytes
 ' from master, if not return to start
   If Temp1 <> Sync Then
      Goto _start
   Else
      ' Get packet in binary mode
      Inputbin Hdb2 , Hdb1 , Dab1 , Sab1 , Db1 , Crc2 , Crc1

   ' Packet header check routine
      ' Check HDB2 to see if MCU are capable to use the packet
      ' structure, if not goto Start
      If Hdb2 <> Chdb2 Then
         Goto _start
      End If

      ' Check HDB1 to see if MCu are capable to use the packet
      ' structure, if not goto Start
      Select Case Hdb1                              ' Check if only receiving data
         Case Chdb1w : Commands = Db1
         Case Chdb1r : Commands = 0
         Case Else : Goto _start
      End Select

      ' Address check routine
      ' Check if this is the node addressed, if not goto Start
      If Dab1 <> Myaddress Then
         Goto _start
      End If

      'keep in mind his address
      Hisaddress = Sab1

      ' CRC check routine
      ' Calculate CRC for all received bytes
      Crc = 0
      Temp2 = Hdb2
      Gosub Calc_crc
      Temp2 = Hdb1
      Gosub Calc_crc
      Temp2 = Dab1
      Gosub Calc_crc
      Temp2 = Sab1
      Gosub Calc_crc
      Temp2 = Db1
      Gosub Calc_crc
      Temp2 = Crc2
      Gosub Calc_crc
      Temp2 = Crc1
      Gosub Calc_crc

      ' Check if there was any CRC errors, if so send NAK
      If Crc <> 0 Then
         Gosub _nak
      End If

      ' No CRC errors in packet so check what to do.
      ' Here write data to PortC or according commands
      Datas = Db1
      Gosub _ack

      If Commands = 0 Then
         Ledport = Datas Xor &B11111111
      Else
         Portb = Commands Xor &B11111111
         Select Case Commands
         Case 128 : Waitms 50
                    Db1 = Tcnt1l
                    Gosub _send

         Case 129 : Enable Timer1
         Case 130 : Disable Timer1

         Case Else : Gosub _ack
         End Select
      End If

' Done, go back to Start and wait for a new packet
  Goto _start
End If

' End of the main program
' -----[ End ]--------------------------------------------------------



' -----[ Subroutines ]------------------------------------------------
'
' Subroutine for calculating CRC value in variable Temp2
Calc_crc:
         Temp_crc = Temp2 * 256
         Crc = Temp_crc Xor Crc
         For Temp1 = 0 To 7
             If Crc > 32767 Then
                Crc = Crc * 2
                Crc = Crc Xor Crcpoly
             Else
                Crc = Crc * 2
             End If
             Crc = Crc And 65535
         Next
Return


' Send Ack (i.e Tell Master That Packet Was Ok)
_ack:
      ' Set ACK bits to ACK in HDB2 (logic XOR bit0 and bit1)
      Db1 = 0
      Hdb2 = Hdb2 Xor &B00000011
      Gosub _send
Return


' Send NAK (i.e tell master that packet was bad)
_nak:
      ' Set ACK bits to NAK in HDB2 (logic OR bit0 and bit1)
      Hdb2 = Hdb2 Or &B00000011
      Gosub _send
Return

' Subroutine for sending values
_send:
      ' Change SAB1 and DAB1 address bytes
      Sab1 = Myaddress
      Dab1 = Hisaddress
      ' Clear CRC variable
      Crc = 0
      ' Put HDB2 in tempory variable
      Temp2 = Hdb2
      ' Calculate CRC
      Gosub Calc_crc

      ' Put HDB1 in tempory variable
      Temp2 = Hdb1
      ' Calculate CRC
      Gosub Calc_crc

      ' Put DAB1 in tempory variable
      Temp2 = Dab1
      ' Calculate CRC
      Gosub Calc_crc

      ' Put SAB1 in tempory variable
      Temp2 = Sab1
      ' Calculate CRC
      Gosub Calc_crc

      ' Put DB1 in tempory variable
      Temp2 = Db1
      ' Calculate CRC
      Gosub Calc_crc

      ' Put calculated Hi_CRC value in CRC2
      Crc2 = High(crc)
      ' Put calculated Lo_CRC value in CRC1
      Crc1 = Low(crc)

      ' Give Receiver time to settle before transmitt any data
      Waitms 5

      ' Send packet to master, including the SYNC byte
      Print Chr(preamble) ; Chr(sync) ; Chr(hdb2) ; Chr(hdb1) ; Chr(dab1) ; Chr(sab1) ; Chr(db1) ; Chr(crc2) ; Chr(crc1);

Return


Timer1_isr:
           Timer1 = Timer1_reload
           Db1 = Db1 - 1
           Gosub _send

Return

